@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2005 Double Precision, Inc.
+** Copyright 1998 - 2006 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -37,7 +37,7 @@
 #include	"tcpd/spipe.h"
 #include	"numlib/numlib.h"
 
-static const char rcsid[]="$Id: imaplogin.c,v 1.31 2005/10/01 03:23:44 mrsam Exp $";
+static const char rcsid[]="$Id: imaplogin.c,v 1.32 2006/05/28 15:29:52 mrsam Exp $";
 
 FILE *debugfile=0;
 extern void mainloop();
@@ -59,7 +59,8 @@ static const char *defaultmaildir;
 
 void rfc2045_error(const char *p)
 {
-	write(2, p, strlen(p));
+	if (write(2, p, strlen(p)) < 0)
+		_exit(1);
 	_exit(0);
 }
 
@@ -112,8 +113,7 @@ int	waitstat;
 		/* couriertls will have the socket on fd 0,
 		** and dup stderr on fd 1 */
 
-		close(1);
-		dup(2);
+		dup2(2, 1);
 		alarm(0);
 		execl( getenv("COURIERTLS"), "couriertls",
 			buf1, "-tcpd", "-server", (char *)0);
@@ -133,7 +133,9 @@ int	waitstat;
 		exit(1);
 	}
 	close(pipefd[0]);
-	write(1, "", 1);	/* child - exec OK now */
+	if (write(1, "", 1) < 0)	/* child - exec OK now */
+		; /* make gcc shut up */
+
 	while (wait(&waitstat) != p)
 		;
 
