@@ -418,7 +418,7 @@ SSL_CTX *tls_create(int isserver, const struct tls_info *info)
 
 	ctx=SSL_CTX_new(protocol && strcmp(protocol, "SSL2") == 0
 							? SSLv2_method():
-		protocol && strcmp(protocol, "SSL3") == 0 ? SSLv23_method():
+		protocol && strcmp(protocol, "SSL3") == 0 ? SSLv3_method():
 		TLSv1_method());
 
 	if (!ctx)
@@ -691,7 +691,8 @@ static int get_func(void *rec, size_t recsize,
 
 	sess=recp + sizeof(timeout) + sizeof(session_id_len) + session_id_len;
 
-	wi->ret=d2i_SSL_SESSION(NULL, &sess, recsize - sizeof(timeout) -
+	wi->ret=d2i_SSL_SESSION(NULL, (const unsigned char **)
+				&sess, recsize - sizeof(timeout) -
 				sizeof(session_id_len) - session_id_len);
 
 	*wi->copyflag=0;
