@@ -52,7 +52,7 @@
 
 #include	<netdb.h>
 
-static const char rcsid[]="$Id: tcpd.c,v 1.43 2006/05/28 15:29:52 mrsam Exp $";
+static const char rcsid[]="$Id: tcpd.c,v 1.44 2006/11/04 20:09:47 mrsam Exp $";
 
 static const char *accessarg=0;
 static const char *accesslocal=0;
@@ -1597,6 +1597,7 @@ int	ipcnt, ccnt;
 char	buf[RFC1035_MAXNAMESIZE+128];
 struct blocklist_s *bl;
 const char *remoteinfo;
+const char *p;
 
 	i=sizeof(lsin);
 	if (sox_getsockname(fd, (struct sockaddr *)&lsin, &i) ||
@@ -1623,6 +1624,15 @@ const char *remoteinfo;
 		putenv(q);
 	}
 
+/* check if it's an exception to the global ip limit */
+	if( (p=getenv("MAXCPERIP")) != NULL )
+	{
+		int j = atoi(p);
+
+		if( j > 0 )
+			maxperip = j;
+	}
+
 	for (i=0, ipcnt=ccnt=0; i<nprocs; i++)
 	{
 	RFC1035_ADDR *psin;
@@ -1638,7 +1648,11 @@ const char *remoteinfo;
 
 		if (j >= sizeof(*addr) &&
 			++ipcnt >= maxperip)
+		{
+			rfc1035_ntoa(addr, buf);
+			fprintf(stderr,"ALERT: Maximum connection limit reached for %s\n",buf);
 			_exit(0);	/* Too many from same IP address */
+		}
 
 		if ( j >= sizeof(*addr)-1 &&
 			++ccnt >= maxperc)
