@@ -1,11 +1,11 @@
 /*
-** Copyright 1998 - 2005 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2007 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: html.c,v 1.26 2005/12/25 21:59:35 mrsam Exp $
+** $Id: html.c,v 1.28 2007/05/20 00:04:55 mrsam Exp $
 */
 #include	<stdio.h>
 #include	<ctype.h>
@@ -16,6 +16,8 @@
 #include	"sqwebmail.h"
 #include	"rfc2045/rfc2045.h"
 
+#include	"html.h"
+
 void decodehtmlchar(char *p)
 {
 char	*q;
@@ -97,36 +99,22 @@ char	*q;
 	Everything in <STYLE>   </STYLE> is dropped..
 */
 
-char *tagbuf=0;
-size_t tagbufsize=0, tagbuflen;
-
-struct tagattrinfo {
-	char *tagname;
-	size_t tagnamelen;
-	char *tagvalue;
-	size_t tagvaluelen;
-
-	char *atagstart;	/* Entire tag=value location */
-	size_t ataglen;
-	} ;
-
-struct tagattrinfo *tagattr=0;
-size_t tagattrsize=0, tagattrlen;
-
-static void addtagbuf(int c)
+static void addtagbuf(struct htmlfilter_info *info, int c)
 {
-	if (tagbufsize >= 1024)	return;	/* DOS attack - get rid of the tag */
+	if (info->tagbufsize >= 1024)	return;	/* DOS attack - get rid of the tag */
 
-	if (tagbuflen >= tagbufsize)
+	if (info->tagbuflen >= info->tagbufsize)
 	{
-	char	*newtagbuf= tagbuf ? (char *)realloc(tagbuf, tagbufsize+256)
-			:(char *)malloc(tagbufsize+256);
+	char	*newtagbuf= info->tagbuf ? (char *)realloc(info->tagbuf, info->tagbufsize+256)
+			:(char *)malloc(info->tagbufsize+256);
 
-		if (!newtagbuf)	enomem();
-		tagbuf=newtagbuf;
-		tagbufsize += 256;
+		if (!newtagbuf)
+			return;
+
+		info->tagbuf=newtagbuf;
+		info->tagbufsize += 256;
 	}
-	tagbuf[tagbuflen++]=c;
+	info->tagbuf[info->tagbuflen++]=c;
 }
 
 /* Parse the contents of tagbuf into individual attributes.  If argument is
@@ -137,12 +125,12 @@ static void addtagbuf(int c)
 ** The first attribute is -- obviously -- the actual tag.
 */
 
-static size_t parseattr(struct tagattrinfo *tai)
+static size_t parseattr(struct htmlfilter_info *info, struct tagattrinfo *tai)
 {
 size_t	c=0;
 char *p;
 
-	for (p=tagbuf; *p; )
+	for (p=info->tagbuf; *p; )
 	{
 		while (*p && isspace((int)(unsigned char)*p))	p++;
 		if (!*p)	break;
@@ -189,13 +177,13 @@ char *p;
 				if (*p)	p++;
 				else
 				{
-					memset(tagbuf, ' ', strlen(tagbuf));
+					memset(info->tagbuf, ' ', strlen(info->tagbuf));
 				}
 			}
 			else
 			{
 				if (c == 0)
-					memset(tagbuf, ' ', strlen(tagbuf));
+					memset(info->tagbuf, ' ', strlen(info->tagbuf));
 
 				if (tai)
 				{
@@ -222,14 +210,14 @@ char *p;
 	return (c);
 }
 
-static void parsetagbuf()
+static void parsetagbuf(struct htmlfilter_info *info)
 {
 	char *p;
 
-	while ((p=strchr(tagbuf, '<')) != NULL)
+	while ((p=strchr(info->tagbuf, '<')) != NULL)
 		*p=' ';
 
-	for (p=tagbuf; *p; p++)
+	for (p=info->tagbuf; *p; p++)
 	{
 		char *q;
 
@@ -254,19 +242,24 @@ static void parsetagbuf()
 		--p;
 	}
 
-        tagattrlen=parseattr(0);
-        if ( tagattrlen > tagattrsize)
+        info->tagattrlen=parseattr(info, 0);
+        if ( info->tagattrlen > info->tagattrsize)
         {
-        struct tagattrinfo *newta= tagattr ? (struct tagattrinfo *)
-                realloc(tagattr, (tagattrlen+16)*sizeof(*tagattr))
+        struct tagattrinfo *newta= info->tagattr ? (struct tagattrinfo *)
+                realloc(info->tagattr, (info->tagattrlen+16)*sizeof(*info->tagattr))
                 :(struct tagattrinfo *)
-			malloc((tagattrlen+16)*sizeof(*tagattr));
+			malloc((info->tagattrlen+16)*sizeof(*info->tagattr));
 
-                if (!newta)     enomem();
-		tagattrsize=tagattrlen+16;
-		tagattr=newta;
+                if (!newta)
+		{
+			info->tagattrlen=0;
+			return;
+		}
+
+		info->tagattrsize=info->tagattrlen+16;
+		info->tagattr=newta;
         }
-        tagattrlen=parseattr(tagattr);
+        info->tagattrlen=parseattr(info, info->tagattr);
 }
 
 /* See if this attribute is the one we're looking for */
@@ -280,9 +273,9 @@ size_t ll=strlen(l);
 
 /* If this is the tag we're looking for */
 
-static int is_htmltag(const char *l)
+static int is_htmltag(struct htmlfilter_info *info, const char *l)
 {
-	return (tagattrlen ? is_attr(tagattr, l):0);
+	return (info->tagattrlen ? is_attr(info->tagattr, l):0);
 }
 
 /* See if the attribute value starts with what we're looking for */
@@ -306,148 +299,139 @@ static int is_valuestart(const char *v, const char *l)
 	initializes.
 */
 
-enum htmlstate {
-	intext,			/* Initial value.  In plain text */
-	seenlt,			/* Seen < */
-	seenltbang,		/* Seen <! */
-	seenltbangdash,		/* Seen <!- */
-	intag,			/* Seen < but not <!, we're collecting the
-				tag */
+#define skipping() (info->instyletag || info->inscripttag)
 
-	incomment,		/* <!--, in a comment, have not seen any
-				dashes */
-	incommentseendash,	/* In a comment, seen - */
-	incommentseendashdash,	/* In a comment, seen -- */
+static void filtered_tag(struct htmlfilter_info *);
 
-	skiptag			/* Ignore <!tag> */
-	} ;
-
-static enum htmlstate cur_state;
-static unsigned instyletag;
-static unsigned inscripttag;
-
-#define skipping() (instyletag || inscripttag)
-
-static void filtered_tag();
-static void (*htmlfiltered_func)(const char *, size_t);
-static char *(*htmlconvertcid_func)(const char *, void *);
-static void *convertcid_arg;
-
-const char *washlink=0;
-const char *washlinkmailto=0;
-const char *contentbase=0;
-
-void htmlfilter_init( void (*func)(const char *, size_t))
+struct htmlfilter_info *htmlfilter_alloc( void (*func)(const char *, size_t))
 {
-	cur_state=intext;
-	htmlfiltered_func=func;
-	instyletag=0;
-	inscripttag=0;
+	struct htmlfilter_info *info=malloc(sizeof(struct htmlfilter_info));
+
+	if (!info)
+		return NULL;
+
+	memset(info, 0, sizeof(*info));
+
+	info->cur_state=intext;
+	info->htmlfiltered_func=func;
+	info->instyletag=0;
+	info->inscripttag=0;
+	return info;
+}
+
+void htmlfilter_free(struct htmlfilter_info *info)
+{
+	if (info->tagattr)
+		free(info->tagattr);
+	if (info->tagbuf)
+		free(info->tagbuf);
+	free(info);
 }
 
 /* Set prefix to wash HTML links */
 
-void htmlfilter_washlink( const char *p)
+void htmlfilter_washlink(struct htmlfilter_info *info, const char *p)
 {
-	washlink=p;
+	info->washlink=p;
 }
 
-void htmlfilter_contentbase( const char *p)
+void htmlfilter_contentbase(struct htmlfilter_info *info, const char *p)
 {
-	contentbase=p;
+	info->contentbase=p;
 }
 
-void htmlfilter_washlinkmailto( const char *p)
+void htmlfilter_washlinkmailto(struct htmlfilter_info *info, const char *p)
 {
-	washlinkmailto=p;
+	info->washlinkmailto=p;
 }
 
-void htmlfilter_convertcid( char *(*cidfunc)(const char *, void *), void *arg)
+void htmlfilter_convertcid(struct htmlfilter_info *info,
+			   char *(*cidfunc)(const char *, void *), void *arg)
 {
-	htmlconvertcid_func=cidfunc;
-	convertcid_arg=arg;
+	info->htmlconvertcid_func=cidfunc;
+	info->convertcid_arg=arg;
 }
 
-void htmlfilter(const char *p, size_t s)
+void htmlfilter(struct htmlfilter_info *info, const char *p, size_t s)
 {
 size_t	l;
 size_t	start=0;
 
 	for (l=0; l<s; l++)
 	{
-		switch (cur_state)	{
+		switch (info->cur_state)	{
 		case intext:
 			if (p[l] == '>')
 			{
-				(*htmlfiltered_func)(p+start, l-start);
-				(*htmlfiltered_func)("&nbsp;", 6);
+				(*info->htmlfiltered_func)(p+start, l-start);
+				(*info->htmlfiltered_func)("&nbsp;", 6);
 				start=l+1;
 			}
 
 			if (p[l] != '<')	continue;
 			if (!skipping())
-				(*htmlfiltered_func)(p+start, l-start);
+				(*info->htmlfiltered_func)(p+start, l-start);
 					/* Output everything up until the tag */
-			cur_state=seenlt;
-			tagbuflen=0;
+			info->cur_state=seenlt;
+			info->tagbuflen=0;
 			break;
 		case seenlt:
 			if (p[l] == '>')
 			{
-				cur_state=intext;
+				info->cur_state=intext;
 				start=l+1;
 				if (!skipping())
-					(*htmlfiltered_func)("<>", 2);
+					(*info->htmlfiltered_func)("<>", 2);
 						/* Eh? */
 				continue;
 			}
 			if (isspace((int)(unsigned char)p[l]))
 				break;
 			if (p[l] == '!')
-				cur_state=seenltbang;
+				info->cur_state=seenltbang;
 			else if (p[l] != '/'
 				 && !isalpha((int)(unsigned char)p[l]))
 			{
 				start=l+1;
-				cur_state=intext;
+				info->cur_state=intext;
 				break;
 			}
 			else
-				cur_state=intag;
-			addtagbuf(p[l]);
+				info->cur_state=intag;
+			addtagbuf(info, p[l]);
 			break;
 		case intag:
 			/* We're in a tag (not a <!-- comment)
 			collect the contents in tagbuf, until > is seen */
 
-			cur_state=intag;
+			info->cur_state=intag;
 			if (p[l] == '>')
 			{
 				start=l+1;
-				cur_state=intext;
-				filtered_tag();	/* Filter this tag */
+				info->cur_state=intext;
+				filtered_tag(info);	/* Filter this tag */
 				continue;
 			}
-			addtagbuf(p[l]);
+			addtagbuf(info, p[l]);
 			continue;
 
 		case skiptag:
 			if (p[l] == '>')
 			{
 				start=l+1;
-				cur_state=intext;
+				info->cur_state=intext;
 			}
 			continue;
 		case seenltbang:
 			/* We have <!.  If - is not here, this is a SGML tag */
 			if (p[l] != '-')
 			{
-				cur_state=skiptag;
+				info->cur_state=skiptag;
 				continue;
 			}
 
-			addtagbuf(p[l]);
-			cur_state=seenltbangdash;
+			addtagbuf(info, p[l]);
+			info->cur_state=seenltbangdash;
 			continue;
 
 		case seenltbangdash:
@@ -458,31 +442,31 @@ size_t	start=0;
 
 			if (p[l] != '-')
 			{
-				cur_state=skiptag;
+				info->cur_state=skiptag;
 				continue;
 			}
 
 			start=l+1;
-			cur_state=incomment;
+			info->cur_state=incomment;
 			continue;
 
 			/* Look for end of comment */
 
 		case incomment:
-			if (p[l] == '-')	cur_state=incommentseendash;
+			if (p[l] == '-')	info->cur_state=incommentseendash;
 			continue;
 		case incommentseendash:
-			cur_state= p[l] == '-' ? incommentseendashdash
+			info->cur_state= p[l] == '-' ? incommentseendashdash
 						:incomment;
 			continue;
 		case incommentseendashdash:
 			if (p[l] == '-')	continue;
 			if (p[l] != '>')
 			{
-				cur_state=incomment;
+				info->cur_state=incomment;
 				continue;
 			}
-			cur_state=intext;
+			info->cur_state=intext;
 			start=l+1;
 			continue;
 		}
@@ -491,10 +475,10 @@ size_t	start=0;
 	/* When we're done with this chunk, if we're doing plain text, or if
 	** we're in a comment, just pass it along */
 
-	switch (cur_state)	{
+	switch (info->cur_state)	{
 	case intext:
 		if (!skipping())
-			(*htmlfiltered_func)(p+start, l-start);
+			(*info->htmlfiltered_func)(p+start, l-start);
 	default:
 		break;
 	}
@@ -513,88 +497,117 @@ static char *redirectencode(const char *, size_t );
 
 /* replacelink takes care of replacing the contents of one tag's value. */
 
-static void replacelink(size_t l, const char *p)
+static void replacelink(struct htmlfilter_info *info, size_t l, const char *p)
 {
-struct tagattrinfo *tagattrp=tagattr+l;
+struct tagattrinfo *tagattrp=info->tagattr+l;
 char	*newbuf;
-size_t	plen=tagattrp->tagvalue - tagbuf;
+size_t	plen=tagattrp->tagvalue - info->tagbuf;
 size_t	i;
 
-	newbuf=malloc(strlen(tagbuf)+strlen(p)+1);
+	newbuf=malloc(strlen(info->tagbuf)+strlen(p)+1);
 			/* Yes, that's a bit too much.  That's OK */
 
-	if (!newbuf)	enomem();
-	memcpy(newbuf, tagbuf, plen);
+	if (!newbuf)
+	{
+		info->tagbuflen=0;
+		addtagbuf(info, 0);
+		return;
+	}
+
+	memcpy(newbuf, info->tagbuf, plen);
 	strcpy(newbuf+plen, p);
 	strcat(newbuf, tagattrp->tagvalue+tagattrp->tagvaluelen);
 
-	tagbuflen=0;
+	info->tagbuflen=0;
 	for (i=0; newbuf[i]; i++)
-		addtagbuf(newbuf[i]);
-	addtagbuf(0);
-	parsetagbuf();
+		addtagbuf(info, newbuf[i]);
+	addtagbuf(info, 0);
+	parsetagbuf(info);
 	free(newbuf);
 }
-
-static void dowashlink(size_t l, const char *value)
+static void dowashlink(struct htmlfilter_info *info,
+		       size_t l, const char *value)
 {
-char	*url, *p;
+	char	*url, *p;
 
 	url=redirectencode(value, strlen(value));
 
-	p=malloc(strlen(url)+strlen(washlink)+1);
-	if (!p)	enomem();
-	strcat(strcpy(p, washlink), url);
-	replacelink(l, p);
+	p=url ? malloc(strlen(url)+strlen(info->washlink)+1):NULL;
+
+	if (!p)
+	{
+		info->tagbuflen=0;
+		addtagbuf(info, 0);
+		return;
+	}
+
+	strcat(strcpy(p, info->washlink), url);
+	replacelink(info, l, p);
 	free(p);
 	free(url);
 }
 
-static void dowashlinkmailto(size_t l, const char *mailtolink)
+static void dowashlinkmailto(struct htmlfilter_info *info,
+			     size_t l, const char *mailtolink)
 {
-size_t mailtolinklen=strlen(mailtolink);
-char	*newlink=malloc(strlen(washlinkmailto)+1+mailtolinklen);
-char	*p;
+	size_t mailtolinklen=strlen(mailtolink);
+	char	*newlink;
+	char	*p;
 
-	if (!newlink)	enomem();
-	strcpy(newlink, washlinkmailto);
+	newlink=malloc(strlen(info->washlinkmailto)+1+mailtolinklen);
+	if (!newlink)
+	{
+		info->tagbuflen=0;
+		addtagbuf(info, 0);
+		return;
+	}
+
+	strcpy(newlink, info->washlinkmailto);
 	strncat(newlink, mailtolink, mailtolinklen);
-	if ((p=strchr(newlink+strlen(washlinkmailto), '?')) != 0)
+	if ((p=strchr(newlink+strlen(info->washlinkmailto), '?')) != 0)
 		*p='&';
-	replacelink(l, newlink);
+	replacelink(info, l, newlink);
 	free(newlink);
 }
 
-static void dowashcid(size_t l, const char *link)
+static void dowashcid(struct htmlfilter_info *info, size_t l, const char *link)
 {
 size_t linklen=strlen(link);
 char	*p;
 
 	p=malloc(linklen+1);
-	if (!p)	enomem();
+
+	if (!p)
+	{
+		info->tagbuflen=0;
+		addtagbuf(info, 0);
+	}
 
 	memcpy(p, link, linklen);
 	p[linklen]=0;
 
-	if (!htmlconvertcid_func)
+	if (!info->htmlconvertcid_func)
 		*p=0;
 	else
 	{
-	char	*q=(*htmlconvertcid_func)(p+4, convertcid_arg);
+		char	*q=(*info->htmlconvertcid_func)(p+4,
+							info->convertcid_arg);
 
 		free(p);
 		p=q;
 	}
-	replacelink(l, p);
-	free(p);
+	replacelink(info, l, p ? p:"");
+	if (p)
+		free(p);
 }
 
 static char *redirectencode(const char *p, size_t l)
 {
-char	*q=malloc(l+1);
-char	*r;
+	char	*q=malloc(l+1);
+	char	*r;
+
+	if (!q)	return NULL;
 
-	if (!q)	enomem();
 	memcpy(q, p, l);
 	q[l]=0;
 	decodehtmlchar(q);
@@ -604,12 +617,12 @@ char	*r;
 }
 
 
-size_t find_tag(const char *tagname)
+static size_t find_tag(struct htmlfilter_info *info, const char *tagname)
 {
 size_t	l;
 
-	for (l=1; l<tagattrlen; l++)
-		if (is_attr(tagattr+l, tagname))	return (l);
+	for (l=1; l<info->tagattrlen; l++)
+		if (is_attr(info->tagattr+l, tagname))	return (l);
 	return (0);
 }
 
@@ -617,72 +630,72 @@ size_t	l;
 	Decide what to do with this tag
 */
 
-static void filtered_tag()
+static void filtered_tag(struct htmlfilter_info *info)
 {
-size_t	l;
-int	open_window=0;
+	size_t	l;
+	int	open_window=0;
  
-	addtagbuf(0);
-	parsetagbuf();
+	addtagbuf(info, 0);
+	parsetagbuf(info);
 
-	if ( is_htmltag("SCRIPT"))
+	if ( is_htmltag(info, "SCRIPT"))
 	{
-		++inscripttag;
+		++info->inscripttag;
 		return;
 	}
 
-	if ( is_htmltag("/SCRIPT"))
+	if ( is_htmltag(info, "/SCRIPT"))
 	{
-		if (inscripttag)
-			--inscripttag;
+		if (info->inscripttag)
+			--info->inscripttag;
 		return;
 	}
 
-	if (	is_htmltag("TITLE") || is_htmltag("/TITLE") ||
-		is_htmltag("FRAME") || is_htmltag("/FRAME") ||
-		is_htmltag("IFRAME") || is_htmltag("/IFRAME") ||
-		is_htmltag("APP") || is_htmltag("/APP") ||
-		is_htmltag("APPLET") || is_htmltag("/APPLET") ||
-		is_htmltag("SERVER") || is_htmltag("/SERVER") ||
-		is_htmltag("OBJECT") || is_htmltag("/OBJECT") ||
-		is_htmltag("HTML") || is_htmltag("/HTML") ||
-		is_htmltag("HEAD") || is_htmltag("/HEAD") ||
-		is_htmltag("BODY") || is_htmltag("/BODY") ||
-		is_htmltag("LINK") || is_htmltag("META"))
+	if (	is_htmltag(info, "TITLE") || is_htmltag(info, "/TITLE") ||
+		is_htmltag(info, "FRAME") || is_htmltag(info, "/FRAME") ||
+		is_htmltag(info, "IFRAME") || is_htmltag(info, "/IFRAME") ||
+		is_htmltag(info, "APP") || is_htmltag(info, "/APP") ||
+		is_htmltag(info, "APPLET") || is_htmltag(info, "/APPLET") ||
+		is_htmltag(info, "SERVER") || is_htmltag(info, "/SERVER") ||
+		is_htmltag(info, "OBJECT") || is_htmltag(info, "/OBJECT") ||
+		is_htmltag(info, "HTML") || is_htmltag(info, "/HTML") ||
+		is_htmltag(info, "HEAD") || is_htmltag(info, "/HEAD") ||
+		is_htmltag(info, "BODY") || is_htmltag(info, "/BODY") ||
+		is_htmltag(info, "LINK") || is_htmltag(info, "META"))
 	{
 		return;
 	}
 
-	if ( is_htmltag("STYLE"))
+	if ( is_htmltag(info, "STYLE"))
 	{
-		++instyletag;
+		++info->instyletag;
 		return;
 	}
 
-	if ( is_htmltag("/STYLE"))
+	if ( is_htmltag(info, "/STYLE"))
 	{
-		if (instyletag)
-			--instyletag;
+		if (info->instyletag)
+			--info->instyletag;
 		return;
 	}
 
 	if (skipping())	return;
 
 
-	for (l=1; l<tagattrlen; l++)
+	for (l=1; l<info->tagattrlen; l++)
 	{
-		if (tagattr[l].tagnamelen > 2 &&
-			strncasecmp(tagattr[l].tagname, "ON", 2) == 0)
+		if (info->tagattr[l].tagnamelen > 2 &&
+			strncasecmp(info->tagattr[l].tagname, "ON", 2) == 0)
 		{
-			memset(tagattr[l].atagstart, ' ',
-				tagattr[l].ataglen);
+			memset(info->tagattr[l].atagstart, ' ',
+				info->tagattr[l].ataglen);
 		}
 	}
 
-	if (is_htmltag("IMG"))
+	if (is_htmltag(info, "IMG"))
 	{
-	size_t	nsrc=find_tag("src");
-	size_t	nalt=find_tag("alt");
+	size_t	nsrc=find_tag(info, "src");
+	size_t	nalt=find_tag(info, "alt");
 	int	ignore_img=0;
 
 		/*
@@ -690,10 +703,10 @@ int	open_window=0;
 		** the URL being processed.  This is handled later.
 		*/
 
-		if (nsrc && htmlconvertcid_func)
+		if (nsrc && info->htmlconvertcid_func)
 		{
-		const char *p=tagattr[nsrc].tagvalue;
-		size_t s=tagattr[nsrc].tagvaluelen;
+		const char *p=info->tagattr[nsrc].tagvalue;
+		size_t s=info->tagattr[nsrc].tagvaluelen;
 
 			while (s)
 			{
@@ -716,12 +729,14 @@ int	open_window=0;
 		char	*q;
 		char	*alt=0;
 
-			r=malloc(tagattr[nsrc].tagvaluelen+1);
-			if (!r)	enomem();
-			memcpy(r, tagattr[nsrc].tagvalue,
-				tagattr[nsrc].tagvaluelen);
-			r[tagattr[nsrc].tagvaluelen]=0;
-			q=rfc2045_append_url(contentbase, r);
+			r=malloc(info->tagattr[nsrc].tagvaluelen+1);
+			if (!r)
+				return;
+
+			memcpy(r, info->tagattr[nsrc].tagvalue,
+				info->tagattr[nsrc].tagvaluelen);
+			r[info->tagattr[nsrc].tagvaluelen]=0;
+			q=rfc2045_append_url(info->contentbase, r);
 			free(r);
 
 			for (r=q; *r; r++)
@@ -731,30 +746,32 @@ int	open_window=0;
 
 			if (nalt)
 			{
-				alt=malloc(tagattr[nalt].tagvaluelen+1);
-				if (!alt)	enomem();
-				memcpy(alt, tagattr[nalt].tagvalue,
-					tagattr[nalt].tagvaluelen);
-				alt[tagattr[nalt].tagvaluelen]=0;
+				alt=malloc(info->tagattr[nalt].tagvaluelen+1);
+				if (!alt)
+					return;
+
+				memcpy(alt, info->tagattr[nalt].tagvalue,
+					info->tagattr[nalt].tagvaluelen);
+				alt[info->tagattr[nalt].tagvaluelen]=0;
 			}
 
-			(*htmlfiltered_func)("<a target=\"_blank\" href=\"", 25);
+			(*info->htmlfiltered_func)("<a target=\"_blank\" href=\"", 25);
 
-			if (washlink)	dowashlink(nsrc, q);
-			else	replacelink(nsrc, q);
+			if (info->washlink)	dowashlink(info, nsrc, q);
+			else	replacelink(info, nsrc, q);
 
-			(*htmlfiltered_func)(tagattr[nsrc].tagvalue,
-					tagattr[nsrc].tagvaluelen);
-			(*htmlfiltered_func)("\">", 2);
+			(*info->htmlfiltered_func)(info->tagattr[nsrc].tagvalue,
+					info->tagattr[nsrc].tagvaluelen);
+			(*info->htmlfiltered_func)("\">", 2);
 			if (alt)
-				(*htmlfiltered_func)(alt, strlen(alt));
+				(*info->htmlfiltered_func)(alt, strlen(alt));
 			else
 			{
-				(*htmlfiltered_func)("[", 1);
-				(*htmlfiltered_func)(q, strlen(q));
-				(*htmlfiltered_func)("]", 1);
+				(*info->htmlfiltered_func)("[", 1);
+				(*info->htmlfiltered_func)(q, strlen(q));
+				(*info->htmlfiltered_func)("]", 1);
 			}
-			(*htmlfiltered_func)("</a>", 4);
+			(*info->htmlfiltered_func)("</a>", 4);
 			free(q);
 			if (alt)	free(alt);
 		}
@@ -765,34 +782,35 @@ int	open_window=0;
 
 	/* Attempt to automatically plug in any holes */
 
-	for (l=1; l<tagattrlen; l++)
+	for (l=1; l<info->tagattrlen; l++)
 	{
-		if (is_attr(tagattr+l, "target") ||
-			is_attr(tagattr+l, "code") ||
-			is_attr(tagattr+l, "language") ||
-			is_attr(tagattr+l, "action") ||
-		    is_attr(tagattr+l, "background") ||
-		    is_attr(tagattr+l, "style") ||
-			(is_attr(tagattr+l, "type")
-			 && !is_htmltag("BLOCKQUOTE")) ||
-			is_attr(tagattr+l, "codetype"))
-			memset(tagattr[l].atagstart, ' ',
-				tagattr[l].ataglen);
+		if (is_attr(info->tagattr+l, "target") ||
+			is_attr(info->tagattr+l, "code") ||
+			is_attr(info->tagattr+l, "language") ||
+			is_attr(info->tagattr+l, "action") ||
+		    is_attr(info->tagattr+l, "background") ||
+		    is_attr(info->tagattr+l, "style") ||
+			(is_attr(info->tagattr+l, "type")
+			 && !is_htmltag(info, "BLOCKQUOTE")) ||
+			is_attr(info->tagattr+l, "codetype"))
+			memset(info->tagattr[l].atagstart, ' ',
+				info->tagattr[l].ataglen);
 
-		if (is_attr(tagattr+l, "href")
-			|| is_attr(tagattr+l, "src")
-			|| is_attr(tagattr+l, "lowsrc"))
+		if (is_attr(info->tagattr+l, "href")
+			|| is_attr(info->tagattr+l, "src")
+			|| is_attr(info->tagattr+l, "lowsrc"))
 		{
-		char	*p=malloc(tagattr[l].tagvaluelen+1), *q;
+		char	*p=malloc(info->tagattr[l].tagvaluelen+1), *q;
 		size_t	n;
 		int	goodhref=0;
 
-			if (!p)	enomem();
+			if (!p)
+				return;
 
-			memcpy(p, tagattr[l].tagvalue, tagattr[l].tagvaluelen);
-			p[tagattr[l].tagvaluelen]=0;
+			memcpy(p, info->tagattr[l].tagvalue, info->tagattr[l].tagvaluelen);
+			p[info->tagattr[l].tagvaluelen]=0;
 
-			q=rfc2045_append_url(contentbase, p);
+			q=rfc2045_append_url(info->contentbase, p);
 			free(p);
 
 			for (p=q; *p; p++)
@@ -806,30 +824,32 @@ int	open_window=0;
 					if (is_valuestart(q, "cid:"))
 					{
 						goodhref=1;
-						dowashcid(l, q);
+						dowashcid(info, l, q);
 					}
 					else if ((is_valuestart(q, "http:") ||
 						is_valuestart(q, "https:")) &&
-						is_attr(tagattr+l, "href"))
+						is_attr(info->tagattr+l, "href"))
 /* block src/lowsrc tags in anything but IMG.
 Ex: <input type="image" src="http://"> -- don't render as a redirect
 URL
 */
 					{
 						goodhref=1;
-						if (washlink)
-							dowashlink(l, q);
+						if (info->washlink)
+							dowashlink(info, l, q);
 						else
-							replacelink(l, q);
-						if (is_htmltag("A"))
+							replacelink(info, l, q);
+						if (is_htmltag(info, "A"))
 							open_window=1;
 						break;
 					}
 					else if ( is_valuestart(q, "mailto:"))
 					{
 						goodhref=1;
-						dowashlinkmailto(l, strchr(q,
-							':')+1);
+						dowashlinkmailto(info,
+								 l, strchr(q,
+									   ':')
+								 +1);
 					}
 					else if ( is_valuestart(q, "ftp:") ||
 						is_valuestart(q, "gopher:") ||
@@ -837,24 +857,24 @@ URL
 						is_valuestart(q, "telnet:"))
 					{
 						goodhref=1;
-						if (is_htmltag("A"))
+						if (is_htmltag(info, "A"))
 							open_window=1;
 					}
 					break;
 				}
 			if (!goodhref)
 			{
-				memset(tagattr[l].atagstart, ' ',
-					tagattr[l].ataglen);
+				memset(info->tagattr[l].atagstart, ' ',
+					info->tagattr[l].ataglen);
 			}
 			free(q);
 		}
 	}
 
-	(*htmlfiltered_func)("<", 1);
-	(*htmlfiltered_func)(tagbuf, strlen(tagbuf));
+	(*info->htmlfiltered_func)("<", 1);
+	(*info->htmlfiltered_func)(info->tagbuf, strlen(info->tagbuf));
 	if (open_window)
-		(*htmlfiltered_func)(" target=\"_blank\"", 16);
-	(*htmlfiltered_func)(">", 1);
+		(*info->htmlfiltered_func)(" target=\"_blank\"", 16);
+	(*info->htmlfiltered_func)(">", 1);
 
 }
