@@ -5,7 +5,7 @@
 
 
 /*
-** $Id: newmsg.c,v 1.62 2006/07/30 19:01:42 mrsam Exp $
+** $Id: newmsg.c,v 1.65 2007/05/06 21:49:29 mrsam Exp $
 */
 #include	"config.h"
 #include	"sqwebmail.h"
@@ -70,6 +70,12 @@ extern int ishttps();
 static void newmsg_header(const char *label, const char *field,
 			  const char *encoded, const char *val)
 {
+int		hdrmaxlen=512;
+const char	*p=getarg("HDRMAXLEN");
+
+	if (p && (atoi(p) > hdrmaxlen))
+		hdrmaxlen=atoi(p);
+
 #if	HAVE_SQWEBMAIL_UNICODE
 const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
 #endif
@@ -78,9 +84,9 @@ const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
 	       "<td width=\"6\">&nbsp;</td>",
 	       field, label);
 
-	printf("<td><input type=\"text\" name=\"%s\" size=\"50\" maxlength=\"512\""
+	printf("<td><input type=\"text\" name=\"%s\" size=\"50\" maxlength=\"%d\""
 	       " class=\"new-message-header-text\" value=\"",
-		field);
+		field, hdrmaxlen);
 	if (encoded)
 	{
 	char	*s;
@@ -118,14 +124,20 @@ static void newmsg_header_rfc822(const char *label, const char *field,
 				 const char *encoded, const char *val,
 				 int is_readonly)
 {
+int		hdrmaxlen=512;
+const char	*p=getarg("HDRMAXLEN");
+
+	if (p && (atoi(p) > hdrmaxlen))
+		hdrmaxlen=atoi(p);
+
 	printf("<tr><th align=\"right\"><p class=\"new-message-header\">"
 	       "<span class=\"new-message-header-%s\">%s</span></p></th>"
 	       "<td width=\"6\">&nbsp;</td>",
 	       field, label);
 
-	printf("<td><input type=\"text\" name=\"%s\" size=\"50\" maxlength=\"512\""
+	printf("<td><input type=\"text\" name=\"%s\" size=\"50\" maxlength=\"%d\""
 	       " class=\"new-message-header-text\" value=\"",
-		field);
+		field, hdrmaxlen);
 	if (encoded)
 	{
 		struct rfc822t *t=rfc822t_alloc_new(encoded, NULL, NULL);
@@ -178,88 +190,6 @@ void newmsg_hiddenheader(const char *label, const char *value)
 
 /* ---------------------------------------------------- */
 
-/* Display message preview */
-
-static void preview_show_func_s(const char *p, size_t n)
-{
-	if (fwrite(p, 1, n, stdout) != n)
-		; /* ignore */
-}
-
-void preview_start()
-{
-	printf("<pre class=\"message-text-plain\">");
-	filter_start(FILTER_FOR_PREVIEW, &preview_show_func_s);
-}
-
-int preview_callback(const char *ptr, size_t cnt, void *voidptr)
-{
-        filter(ptr, cnt);
-        return (0);
-}
-
-void preview_end()
-{
-	filter_end();
-	printf("</pre>\n");
-}
-
-#if 0
-static void show_preview(const char *filename)
-{
-char	*header, *value;
-struct	rfc2045	*rfcp;
-FILE	*fp;
-int	fd;
-
-	fp=0;
-	fd=maildir_safeopen(filename, O_RDONLY, 0);
-	if (fd >= 0)
-		if ((fp=fdopen(fd, "r")) == 0)
-			close(fd);
-
-	if (!fp)	return;
-
-	while ((header=maildir_readheader(fp, &value, 1)) != 0)
-	{
-		/* Don't show X-, From, and Content- headers in preview */
-
-		if (strncmp(header, "x-", 2) == 0)	continue;
-		if (strcmp(header, "mime-version") == 0)	continue;
-		if (strncmp(header, "content-", 8) == 0)	continue;
-
-		printf("%c", toupper(*header));
-		output_attrencoded_oknl(header+1);
-		printf(": ");
-
-		value=rfc2047_decode_enhanced(value, sqwebmail_content_charset);
-		if (value)
-		{
-			output_attrencoded_oknl(value);
-			free(value);
-		}
-		printf("\n");
-	}
-	printf("\n");
-
-	rfcp=rfc2045_fromfp(fp);
-	if (!rfcp)	return;
-
-	filter_start(FILTER_FOR_PREVIEW, &preview_show_func_s);
-	{
-		struct rfc2045 *q=
-			rfc2045_searchcontenttype(rfcp, "text/plain");
-
-		if (q)
-			rfc2045_decodemimesection(fileno(fp), q,
-						  &filter_stub, NULL);
-	}
-	rfc2045_free(rfcp);
-	filter_end();
-}
-#endif
-
-
 /*
 ** Strip leading space from flowed text format-encoded message.
 */
@@ -947,7 +877,7 @@ int dsn;
 		close(pipefd1[1]);
 
 		if (dsn)
-			putenv("DSN=-Nsuccess,delay,fail");
+			putenv("DSN=-Nsuccess,delay,failure");
 		else
 			putenv("DSN=");
 
