@@ -5,7 +5,7 @@
 
 
 /*
-** $Id: newmsg_create.c,v 1.58 2005/11/20 00:04:51 mrsam Exp $
+** $Id: newmsg_create.c,v 1.60 2007/05/07 00:31:54 mrsam Exp $
 */
 #include	"config.h"
 #include	"cgi/cgi.h"
@@ -305,7 +305,7 @@ char	*q;
 	}
 }
 
-static void do_save(const char *, size_t);
+static void do_save(const char *, size_t, void *);
 static int do_save2(const char *, int, void *);
 
 char *newmsg_createdraft_do(const char *curdraft, const char *newmsg,
@@ -563,10 +563,16 @@ char *buf;
 		while (i > 2 && strncmp(buf+i-2, "\n", 1) == 0)
 			--i;
 		buf[i-1]=0;
-		filter_start(FILTER_FOR_SAVING, &do_save);
-		filter(buf, i-1);
-		filter("\n", 1);
-		filter_end();
+
+		{
+			struct filter_info save_filter_info;
+
+			filter_start(&save_filter_info,
+				     FILTER_FOR_SAVING, &do_save, NULL);
+			filter(&save_filter_info, buf, i-1);
+			filter(&save_filter_info, "\n", 1);
+			filter_end(&save_filter_info);
+		}
 	}
 
 	if (pref_wikifmt)
@@ -667,7 +673,7 @@ char *buf;
 	return(draftfilename);
 }
 
-static void do_save(const char *p, size_t l)
+static void do_save(const char *p, size_t l, void *dummy)
 {
 	maildir_writemsg(newdraftfd, p, l);
 }
@@ -684,7 +690,17 @@ static void sentmsg_copy(FILE *, struct rfc2045 *);
 static void sentmsg_reformat(FILE *, struct rfc2045 *, int,
 			     const char *);
 
-extern void header_uc(char *h);
+static void header_uc(char *h)
+{
+	while (*h)
+	{
+		*h=toupper( (int)(unsigned char) *h);
+		while (*h)
+		{
+			if (*h++ == '-')	break;
+		}
+	}
+}
 
 struct lookup_buffers {
 	struct lookup_buffers *next;
