@@ -5,7 +5,7 @@
 
 
 /*
-** $Id: sqwebmail.c,v 1.138 2006/07/25 01:21:17 mrsam Exp $
+** $Id: sqwebmail.c,v 1.142 2007/03/05 04:43:24 mrsam Exp $
 */
 #include	"sqwebmail.h"
 #include	"sqconfig.h"
@@ -41,9 +41,7 @@
 #include	<sys/types.h>
 #include	<sys/stat.h>
 #include	<sys/types.h>
-#include	<sys/stat.h>
 #include        <sys/socket.h>
-#include        <sys/un.h>
 #if	HAVE_FCNTL_H
 #include	<fcntl.h>
 #endif
@@ -92,18 +90,6 @@ extern char *crypt(const char *, const char *);
 #endif
 #include	"htmllibdir.h"
 
-#define PASSFD 0
-
-#if PASSFD_MSGACCRIGHTS
-#undef PASSFD
-#define PASSFD 1
-#endif
-
-#if PASSFD_MSGCONTROL
-#undef PASSFD
-#define PASSFD 1
-#endif
-
 #include	"logindomainlist.h"
 
 #include	"strftime.h"
@@ -154,7 +140,7 @@ extern int nochangepass();
 
 /* Need to cache the following environment variables */
 static const char * const authvars[] = { "AUTHADDR", "AUTHFULLNAME",
-					 "OPTIONS", 0 };
+					 "OPTIONS", "AUTHENTICATED", 0 };
 
 #ifdef	GZIP
 static int gzip_save_fd;
@@ -176,6 +162,7 @@ struct template_stack {
 
 static struct template_stack *template_stack=NULL;
 
+char *trim_spaces(const char *s);
 
 size_t get_timeoutsoft()
 {
@@ -2142,9 +2129,7 @@ static int valid_redirect()
 	return 1;
 }
 
-static int main2();
-
-static int read_environ(int);
+static void main2();
 
 static void usage()
 {
@@ -2153,15 +2138,14 @@ static void usage()
 	exit(1);
 }
 
-static int start_daemon();
-
-static void run_daemon(int fd, int termfd, int connfd);
-
-static void run_prefork(int fd, size_t ndaemons);
+static void call_main2(void *dummy)
+{
+	main2();
+	cleanup();
+}
 
 int main(int argc, char **argv)
 {
-	int fd;
 	const char *prefork;
 	int n;
 
@@ -2170,432 +2154,15 @@ int main(int argc, char **argv)
 	if (argc > 1)
 		usage();
 
-	fd=start_daemon();
-
 	prefork=getenv("SQWEBMAIL_PREFORK");
 
 	n=prefork ? atoi(prefork):5;
 
-	if (n > 0)
-		run_prefork(fd, n);
-	else
-		run_daemon(fd, -1, -1);
+	cgi_daemon(n, SOCKFILENAME, NULL, call_main2, NULL);
+
 	return (0);
 }
 
-/* Start in daemon mode.  Return listening socket file descriptor */
-
-static int start_daemon()
-{
-	int     fd;
-	struct  sockaddr_un skun;
-
-	unlink(SOCKFILENAME);
-
-	fd=socket(PF_UNIX, SOCK_STREAM, 0);
-        if (fd < 0)
-	{
-		perror("socket");
-		return (-1);
-	}
-
-        skun.sun_family=AF_UNIX;
-        strcpy(skun.sun_path, SOCKFILENAME);
-        if (bind(fd, (const struct sockaddr *)&skun, sizeof(skun)) ||
-                listen(fd, SOMAXCONN) ||
-                chmod(skun.sun_path, 0777) ||
-                fcntl(fd, F_SETFL, O_NONBLOCK) < 0)
-        {
-                perror(SOCKFILENAME);
-                close(fd);
-                return (-1);
-        }
-	return fd;
-}
-
-static int prefork(int listenfd, int *allpipes, size_t ndaemos,
-		   int *termpipe);
-
-static void run_prefork(int fd, size_t ndaemons)
-{
-	int *cpipes; /* Completion pipes from preforked processes */
-	int termpipe[2]; /* Termination pipe to preforked processes */
-	size_t i;
-
-	if ((cpipes=malloc(sizeof(int)*ndaemons)) == NULL)
-	{
-		fprintf(stderr,
-		       "CRIT: malloc failed: %s\n",
-		       strerror(errno));
-		exit(1);
-	}
-
-	if (pipe(termpipe) < 0)
-	{
-		fprintf(stderr,
-			"CRIT: pipe failed: %s\n",
-			strerror(errno));
-		exit(1);
-	}
-
-
-	/* Start the initial set of preforked daemons */
-
-	for (i=0; i<ndaemons; i++)
-		cpipes[i]= -1;
-
-	for (i=0; i<ndaemons; i++)
-		cpipes[i]=prefork(fd, cpipes, ndaemons, termpipe);
-
-
-	for (;;)
-	{
-		fd_set	fdr;
-		int	maxfd=0;
-
-		FD_ZERO(&fdr);
-
-		for (i=0; i<ndaemons; i++)
-		{
-			if (cpipes[i] >= maxfd)
-				maxfd=cpipes[i]+1;
-
-			FD_SET(cpipes[i], &fdr);
-		}
-
-		if (select(maxfd, &fdr, NULL, NULL, NULL) <= 0)
-			continue;
-
-		/*
-		** When child process gets a connection, it closes its
-		** completion pipe, which makes the pipe selectable for
-		** read.
-		*/
-
-		for (i=0; i<ndaemons; i++)
-		{
-			if (FD_ISSET(cpipes[i], &fdr))
-			{
-				close(cpipes[i]);
-				cpipes[i]= -1;
-				cpipes[i]=prefork(fd, cpipes,
-						  ndaemons, termpipe);
-			}
-		}
-	}
-}
-
-/* Start a preforked process */
-
-static int prefork(int listenfd, int *allpipes, size_t ndaemons,
-		   int *termpipe)
-{
-	int newpipe[2];
-	pid_t p;
-	int waitstat;
-	size_t i;
-
-	if (pipe(newpipe) < 0)
-	{
-		fprintf(stderr,
-			"CRIT: pipe failed: %s\n", strerror(errno));
-		exit(1);
-	}
-
-	while ((p=fork()) < 0)
-	{
-		fprintf(stderr,
-			"CRIT: fork failed: %s\n", strerror(errno));
-		sleep(5);
-	}
-
-	if (p) /* parent */
-	{
-		close(newpipe[1]);
-
-		/* Wait for first child process to go away */
-
-		while (wait(&waitstat) != p)
-			;
-
-		return (newpipe[0]);
-	}
-
-	close(newpipe[0]);
-	close(termpipe[1]);
-
-	/* New child doesn't need pipes from other children */
-
-	for (i=0; i<ndaemons; i++)
-		if (allpipes[i] >= 0)
-			close(allpipes[i]);
-
-	/* Fork once more, so that the parent process can continue */
-
-	if (fork())
-		exit(0);
-
-	run_daemon(listenfd, termpipe[0], newpipe[1]);
-	return (-1);
-}
-
-static void run_daemon(int fd, int termfd, int acceptedfd)
-{
-	int cfd;
-
-	for (;;)
-	{
-		fd_set	fdr;
-		pid_t p;
-		int maxfd;
-
-		FD_ZERO(&fdr);
-
-		FD_SET(fd, &fdr);
-
-		maxfd=fd;
-
-		if (termfd >= 0)
-		{
-			if (termfd > maxfd)
-				maxfd=termfd;
-
-			FD_SET(termfd, &fdr);
-		}
-
-		if (select(maxfd+1, &fdr, NULL, NULL, NULL) <= 0)
-			continue;
-		if (termfd >= 0 &&
-		    FD_ISSET(termfd, &fdr)) /* Terminate all child procs */
-			exit(0);
-
-		if (!FD_ISSET(fd, &fdr))
-			continue;
-
-		cfd=accept(fd, NULL, 0);
-
-		if (cfd < 0)
-			continue;
-
-		if (acceptedfd >= 0) /* preforked daemon */
-		{
-			if (termfd >= 0)
-				close(termfd);
-			close(acceptedfd);
-			break;
-		}
-
-		p=fork();
-
-		if (p < 0)
-		{
-			fprintf(stderr,
-			       "CRIT: fork() failed: %s\n", strerror(errno));
-			continue;
-		}
-
-		if (p)
-		{
-			int dummy;
-
-			close(cfd);
-			while (wait(&dummy) != p)
-				continue;
-			continue;
-		}
-
-		/* Child forks once more, parent exits */
-
-		if (fork())
-			exit(0);
-
-		break;
-
-	}
-
-	/* child */
-
-	if (fcntl(cfd, F_SETFL, 0) < 0)
-	{
-		fprintf(stderr,
-		       "CRIT: fcntl(): %s\n", strerror(errno));
-		exit(0);
-	}
-
-	close(fd);
-	if (read_environ(cfd))
-	{
-		close(0);
-		close(1);
-		if (dup(cfd) != 0 || dup(cfd) != 1)
-		{
-			fprintf(stderr,
-			       "CRIT: dup() did not work as expected\n");
-			exit(0);
-		}
-		close(cfd);
-	}
-
-	if (fcntl(0, F_SETFL, 0) < 0 ||
-	    fcntl(1, F_SETFL, 0) < 0)
-	{
-		fprintf(stderr,
-		       "CRIT: fcntl() failed: %s\n", strerror(errno));
-		exit(0);
-	}
-
-	main2();
-	cleanup();
-	exit(0);
-}
-
-
-/* Read environment from the sqwebmail stub */
-
-static void force_read(int cfd, char *p, size_t l)
-{
-	int m;
-
-	while (l)
-	{
-		m=read(cfd, p, l);
-		if (m <= 0)
-		{
-			fprintf(stderr,
-			       "WARN: socket closed while reading"
-			       " environment.\n");
-			exit(0);
-		}
-
-		p += m;
-		l -= m;
-	}
-}
-
-/* Receive CGI environment */
-
-static int read_environ(int cfd)
-{
-	static char buf[SOCKENVIRONLEN];
-	size_t l;
-	char *p;
-	int passfd;
-
-	force_read(cfd, buf, 1+sizeof(l));
-
-	memcpy(&l, buf, sizeof(l));
-	passfd=buf[sizeof(l)];
-
-	if (l >= sizeof(buf))
-	{
-		fprintf(stderr,
-		       "WARN: invalid environment received via socket.\n");
-		exit(0);
-	}
-
-	alarm(10); /* Just in case - punt */
-	force_read(cfd, buf, l);
-	buf[l]=0;
-	alarm(0);
-
-	/* Vet environment strings for only known good strings */
-
-	p=buf;
-	while (p < buf+l)
-	{
-		if (strchr(p, '=') == NULL ||
-		    !VALIDCGIVAR(p))
-		{
-			fprintf(stderr,
-			       "WARN: invalid environment received"
-			       " via socket: %s\n" , p);
-			exit(0);
-		}
-
-		putenv(p);
-
-		while (*p++)
-			;
-	}
-
-	/* Receive file descriptors, if supported by the platform */
-
-#if	PASSFD
-
-	if (passfd)
-	{
-		struct iovec iov;
-		char dummy;
-
-#if PASSFD_MSGACCRIGHTS
-
-		int fdbuf[2];
-		struct msghdr msg;
-#endif
-
-#if PASSFD_MSGCONTROL
-
-		int fdbuf[2];
-		struct msghdr msg;
-		struct cmsghdr *cmsg;
-		char buf[CMSG_SPACE(sizeof(fdbuf))];
-#endif
-
-#if PASSFD_MSGACCRIGHTS
-		memset(&iov, 0, sizeof(iov));
-		msg.msg_accrights=(caddr_t)fdbuf;
-		msg.msg_accrightslen=sizeof(fdbuf);
-#endif
-
-
-#if PASSFD_MSGCONTROL
-		memset(&msg, 0, sizeof(msg));
-		msg.msg_control=buf;
-		msg.msg_controllen=sizeof(buf);
-#endif
-
-		msg.msg_iov=&iov;
-		msg.msg_iovlen=1;
-
-		iov.iov_base=&dummy;
-		iov.iov_len=1;
-
-		if (recvmsg(cfd, &msg, 0) <= 0)
-			error("Internal error - recvmsg() failed");
-
-#if PASSFD_MSGACCRIGHTS
-
-		if (msg.msg_accrightslen < sizeof(fdbuf))
-			error("Internal error - malformed recvmsg()");
-#endif
-
-#if PASSFD_MSGCONTROL
-		if (msg.msg_controllen < sizeof(buf) ||
-		    (cmsg = CMSG_FIRSTHDR(&msg))->cmsg_level != SOL_SOCKET ||
-		    cmsg->cmsg_type != SCM_RIGHTS ||
-		    cmsg->cmsg_len != CMSG_LEN(sizeof(fdbuf)))
-		{
-			error("Internal error - malformed recvmsg()");
-			return 0;
-		}
-
-		memcpy(fdbuf, CMSG_DATA(cmsg), sizeof(fdbuf));
-#endif
-		close(0);
-		close(1);
-		if (dup(fdbuf[0]) != 0 || dup(fdbuf[1]) != 1)
-			fprintf(stderr,
-			       "CRIT: dup() did not work as expected in"
-			       " read_environ()\n");
-		close(fdbuf[0]);
-		close(fdbuf[1]);
-		return 0;
-	}
-#endif
-
-	return 1;
-}
-
 static int setuidgid(uid_t u, gid_t g, const char *dir, void *dummy)
 {
 	if (setgid(g) || setuid(u))
@@ -2615,7 +2182,7 @@ static int setuidgid(uid_t u, gid_t g, const char *dir, void *dummy)
 	return (0);
 }
 
-static int main2()
+static void main2()
 {
 const char	*u;
 const char	*ip_addr;
@@ -2678,7 +2245,7 @@ time_t	timeouthard=TIMEOUTHARD;
 			setgid(getgid());
 			setuid(getuid());
 			output_form("printnocookie.html");
-			return (0);
+			return;
 		}
 		reset_cookie=1;
 		cgi_setcookie("sqwebmail-pi", "DELETED");
@@ -2765,7 +2332,7 @@ time_t	timeouthard=TIMEOUTHARD;
 			if (chdir("/") < 0)
 			{
 				output_form("expired.html");
-				return (0);
+				return;
 			}
 			cgi_setup();
 			init_default_locale();
@@ -2781,10 +2348,10 @@ time_t	timeouthard=TIMEOUTHARD;
 				*/
 			{
 				http_redirect_top("");
-				return (0);
+				return;
 			}
 			output_form("expired.html");
-			return (0);
+			return;
 		}
 		free(q);
 		cgiformdatatempdir("tmp");
@@ -2803,7 +2370,7 @@ time_t	timeouthard=TIMEOUTHARD;
 			init_default_locale();
 			output_form("expired.html");
 			free(pi);
-			return (0);
+			return;
 		}
 
 		sqwebmail_mailboxid=u;
@@ -2815,7 +2382,7 @@ time_t	timeouthard=TIMEOUTHARD;
 			{
 				output_form("expired.html");
 				free(pi);
-				return (0);
+				return;
 			}
 
 			sqwebmail_homedir_dev=stat_buf.st_dev;
@@ -2892,7 +2459,7 @@ time_t	timeouthard=TIMEOUTHARD;
 		cgi_setup();
 		init_default_locale();
 
-		if (*(u=cgi("username")))
+		if (*(u=trim_spaces(cgi("username"))))
 			/* Request to log in */
 		{
 		const char *p=cgi("password");
@@ -2900,6 +2467,7 @@ time_t	timeouthard=TIMEOUTHARD;
 		const char *u2=cgi("logindomain");
 		char	*ubuf=malloc(strlen(u)+strlen(u2)+2);
 
+			if (ubuf == NULL) enomem();
 			strcpy(ubuf, u);
 			if (*u2)
 				strcat(strcat(ubuf, "@"), u2);
@@ -2999,7 +2567,7 @@ time_t	timeouthard=TIMEOUTHARD;
 						    "&form=folders":
 						    "&form=refreshfr", "", "");
 				free(ubuf);
-				return(0);
+				return;
 			}
 			maildir_cache_cancel();
 
@@ -3007,7 +2575,7 @@ time_t	timeouthard=TIMEOUTHARD;
 			setgid(getgid());
 			setuid(getuid());
 			output_form("invalid.html");	/* Invalid login */
-			return (0);
+			return;
 		}
 
 		setgid(getgid());
@@ -3042,9 +2610,9 @@ time_t	timeouthard=TIMEOUTHARD;
 			http_redirect_top("?index=1");
 		}
 				
-		return (0);
+		return;
 	}
-	return (0);
+	return;
 }
 
 #ifdef	malloc
@@ -3123,3 +2691,28 @@ size_t i;
 	abort();
 }
 #endif
+
+/* Trim leading and trailing white spaces from string */
+
+char *trim_spaces(const char *s)
+{
+	char *p, *q;
+
+	p=strdup(s);
+	if (!p)
+		enomem();
+
+	if (*p)
+	{
+		for (q=p+strlen(p)-1; q >= p && isspace(*q); q--)
+			*q=0;
+
+		for (q=p; *q && isspace(*q); q++)
+			;
+		if (p != q)
+			p=q;
+	}
+
+	return (p);
+}
+
